@@ -1,6 +1,6 @@
 
 /*
- *  $Id: streqv.h,v 2.3 1999/07/07 19:41:00 bkorb Exp $
+ *  $Id: streqv.h,v 2.4 2000/10/07 22:52:08 bkorb Exp $
  *
  *  String Equivalence
  *
@@ -57,23 +57,19 @@
 #define TOOLS_STREQUIVALENCE
 
 #include <sys/types.h>
+#include "options.h"
 
 /* strneqvcmp
  *
  *  Compare a pair of equivalenced strings for a fixed length
  */
-    int
-strneqvcmp( const char* s1,
-            const char* s2,
-            size_t      ct );
+extern int strneqvcmp PROTO(( const char* s1, const char* s2, size_t ct ));
 
 /* streqvcmp
  *
  *  Compare a pair of NUL-terminated equivalenced strings
  */
-    int
-streqvcmp( const char* s1,
-           const char* s2 );
+extern int streqvcmp PROTO(( const char* s1, const char* s2 ));
 
 /* strequate
  *
@@ -81,8 +77,7 @@ streqvcmp( const char* s1,
  *  compare as equivalent (the second and following characters
  *  will be mapped to the first character).
  */
-    void
-strequate( const char* s );
+extern void strequate PROTO(( const char* s ));
 
 /*
  *  streqvmap
@@ -94,18 +89,18 @@ strequate( const char* s );
  *
  *  DEFAULT STATE:  lower case is mapped to upper with nothing else remapped.
  */
-    void
-streqvmap( char chFrom,
-           char chTo,
-           int  ct );
+extern void streqvmap PROTO(( char chFrom, char chTo, int ct ));
 
 /*
  *  strtransform
  *
  *  Transform a string according to the current equivalence map.
  */
-    void
-strtransform( char* d, char* s );
+extern void strtransform PROTO(( char* d, const char* s ));
 
 #endif /* TOOLS_STREQUIVALENCE */
-/* streqv.h ends here */
+/*
+ * Local Variables:
+ * c-file-style: "stroustrup"
+ * End:
+ * streqv.h ends here */
