@@ -1,6 +1,6 @@
 
 /*
- *  usage.c  $Id: usage.c,v 2.6 2000/04/10 14:03:32 bkorb Exp $
+ *  usage.c  $Id: usage.c,v 2.16 2000/10/31 22:16:56 bkorb Exp $
  *
  *  This module implements the default usage procedure for
  *  Automated Options.  It may be overridden, of course.
@@ -60,15 +60,53 @@
 #  include <compat/pathfind.c>
 #endif
 
+tSCC zOptsOnly[]   = "All arguments are named options.\n";
+tSCC zNumberOpt[]  = "The '-#<number>' option may omit the hash char\n";
+tSCC zHomePath[]   = " - reading file /... %s's exe directory .../%s \n";
+tSCC zExamineFmt[] = " - examining environment variables named %s_*\n";
+tSCC zMust[]       = "\t\t\t\t- must appear between %d and %d times\n";
+tSCC zNoLim[]      = "\t\t\t\t- may appear multiple times\n";
+tSCC zPreset[]     = "\t\t\t\t- may NOT appear - preset only\n";
+tSCC zUpTo[]       = "\t\t\t\t- may appear up to %d times\n";
+tSCC zNoPreset[]   = "\t\t\t\t- may not be preset\n";
+tSCC zAlt[]        = "\t\t\t\t- an alternate for %s\n";
+tSCC zEnab[]       = "\t\t\t\t- enabled by default\n";
+tSCC zDis[]        = "\t\t\t\t- disabled as --%s\n";
+tSCC zTabout[]     = "\t\t\t\t%s\n";
+tSCC zTabHyp[]     = "\t\t\t\t- ";
+tSCC zTabHypAnd[]  = "\t\t\t\t-- and ";
+tSCC zDefaultOpt[] = "\t\t\t\t- default option for unnamed options\n";
+tSCC zReqThese[]   = "requires these options:\n";
+tSCC zProhib[]     = "prohibits these options:\n";
 
-    void
-optionUsage( tOptions*  pOptions, int exitCode )
+tSCC zNoReq_Short_Title[]   = "  Flg Arg Option-Name    Description\n";
+tSCC zNoReq_NoShort_Title[] = "   Arg Option-Name    Description\n";
+tSCC zNrmOptFmt[]           = " %1$3s %2$-14s %4$s\n";
+
+tSCC zReq_Short_Title[]     = "  Flg Arg Option-Name   Req?  Description\n";
+tSCC zReq_NoShort_Title[]   = "   Arg Option-Name   Req?  Description\n";
+tSCC zReqOptFmt[]           = " %3s %-14s %-5s%s\n";
+
+tSCC zIntro[]      = "\n\
+The following option preset mechanisms are supported:\n";
+
+tSCC zFlagOkay[]   =
+"Options may be specified by doubled hyphens and their name\n\
+or by a single hyphen and the flag character (option value).\n";
+
+tSCC zNoFlags[]    =
+"Options are specified by their name and either single\n\
+or doubled %ss.  Flag characters are not interpreted.\n";
+
+
+DEF_PROC_2( void optionUsage,
+            tOptions*,  pOptions,
+            int,        exitCode )
 {
-    tSCC  zReqOptFmt[]   = " %3s %-14s %-5s%s\n";
-    tSCC  zNrmOptFmt[]   = " %1$3s %2$-14s %4$s\n";
-    tCC*  pOptFmt;
-    tCC*  pOptTitle;
-    FILE* fp = stderr;
+    tCC*    pOptFmt;
+    tCC*    pOptTitle;
+    FILE*   fp = stderr;
+    ag_bool displayEnum = AG_FALSE;
 
     fprintf( fp, pOptions->pzUsageTitle, pOptions->pzProgName );
 
@@ -77,44 +115,34 @@ optionUsage( tOptions*  pOptions, int exitCode )
      */
     switch (pOptions->fOptSet & (OPTPROC_NO_REQ_OPT | OPTPROC_SHORTOPT)) {
     case (OPTPROC_NO_REQ_OPT | OPTPROC_SHORTOPT):
-    {
-        tSCC zOptTitle[] = "  Flg Arg Option-Name    Description\n";
-
-        pOptTitle = zOptTitle;
+        pOptTitle = zNoReq_Short_Title;
         pOptFmt   = zNrmOptFmt;
         break;
-    }
 
     case OPTPROC_NO_REQ_OPT:
-    {
-        tSCC zOptTitle[] = "   Arg Option-Name    Description\n";
-
-        pOptTitle = zOptTitle;
+        pOptTitle = zNoReq_NoShort_Title;
         pOptFmt   = zNrmOptFmt;
         break;
-    }
 
     case OPTPROC_SHORTOPT:
-    {
-        tSCC zOptTitle[] = "  Flg Arg Option-Name   Req?  Description\n";
-
-        pOptTitle = zOptTitle;
+        pOptTitle = zReq_Short_Title;
         pOptFmt   = zReqOptFmt;
         break;
-    }
 
     default:
     case 0:
-    {
-        tSCC zOptTitle[] = "   Arg Option-Name   Req?  Description\n";
-
-        pOptTitle = zOptTitle;
+        pOptTitle = zReq_NoShort_Title;
         pOptFmt   = zReqOptFmt;
-        break;
-    }
     }
 
-    fputs( pOptTitle, fp );
+    /*
+     *  When we exit with EXIT_SUCCESS and the first option is a doc option,
+     *  we do *NOT* want to emit the column headers.  Otherwise, we do.
+     */
+    if (  (exitCode != EXIT_SUCCESS)
+       || ((pOptions->pOptDesc->fOptState & OPTST_DOCUMENT) == 0) )
+
+        fputs( pOptTitle, fp );
 
     {
         int        ct     = pOptions->optCt;
@@ -123,13 +151,14 @@ optionUsage( tOptions*  pOptions, int exitCode )
         int        docCt  = 0;
 
         do  {
-            tSCC  zReqArg[] = "YES";
-            tSCC  zNumArg[] = "Num";
-            tSCC  zOptArg[] = "opt";
-            tSCC  zNoArg[]  = "no ";
-            tSCC  zBreak[]  = "\n%s\n\n%s";
-            tSCC  zAuto[]   =
-                "Auto-supported Options:";
+            tSCC  zReqArg[]  = "YES";
+            tSCC  zNumArg[]  = "Num";
+            tSCC  zKeyArg[]  = "KWd";
+            tSCC  zBoolArg[] = "T/F";
+            tSCC  zOptArg[]  = "opt";
+            tSCC  zNoArg[]   = "no ";
+            tSCC  zBreak[]   = "\n%s\n\n%s";
+            tSCC  zAuto[]    = "Auto-supported Options:";
 
             tCC*  pzArgType;
 
@@ -171,6 +200,15 @@ optionUsage( tOptions*  pOptions, int exitCode )
              */
             if ((pOD->fOptState & OPTST_NUMERIC) != 0)
                 pzArgType = zNumArg;
+
+            else if ((pOD->fOptState & OPTST_BOOLEAN) != 0)
+                pzArgType = zBoolArg;
+
+            else if ((pOD->fOptState & OPTST_ENUMERATION) != 0) {
+                displayEnum |= (pOD->pOptProc != NULL) ? AG_TRUE : AG_FALSE;
+                pzArgType = zKeyArg;
+            }
+
             else switch (pOD->optArgType) {
             case ':': pzArgType = zReqArg; break;
             case '?': pzArgType = zOptArg; break;
@@ -197,8 +235,6 @@ optionUsage( tOptions*  pOptions, int exitCode )
              */
             if (  (pOD->pOptMust != (int*)NULL)
                || (pOD->pOptCant != (int*)NULL) ) {
-                tSCC zTabout[] = "\t\t\t\t%s\n";
-                tSCC zTabHyp[] = "\t\t\t\t- ";
 
                 fputs( zTabHyp, fp );
 
@@ -206,10 +242,9 @@ optionUsage( tOptions*  pOptions, int exitCode )
                  *  DEPENDENCIES:
                  */
                 if (pOD->pOptMust != (int*)NULL) {
-                    tSCC zReq[]       = "requires these options:\n";
                     const int* pOptNo = pOD->pOptMust;
 
-                    fputs( zReq, fp );
+                    fputs( zReqThese, fp );
                     for (;;) {
                         fprintf( fp, zTabout, pOptions->pOptDesc[
                                  *pOptNo ].pz_Name );
@@ -217,17 +252,14 @@ optionUsage( tOptions*  pOptions, int exitCode )
                             break;
                     }
 
-                    if (pOD->pOptCant != (int*)NULL) {
-                        tSCC zTabHypAnd[] = "\t\t\t\t-- and ";
+                    if (pOD->pOptCant != (int*)NULL)
                         fputs( zTabHypAnd, fp );
-                    }
                 }
 
                 /*
                  *  CONFLICTS:
                  */
                 if (pOD->pOptCant != (int*)NULL) {
-                    tSCC   zProhib[] = "prohibits these options:\n";
                     const int* pOptNo = pOD->pOptCant;
 
                     fputs( zProhib, fp );
@@ -244,10 +276,15 @@ optionUsage( tOptions*  pOptions, int exitCode )
              *  IF there is a disablement string
              *  THEN print the disablement info
              */
-            if (pOD->pz_DisableName != (char*)NULL )  {
-                tSCC zDis[] = "\t\t\t\t- disabled as --%s\n";
+            if (pOD->pz_DisableName != (char*)NULL )
                 fprintf( fp, zDis, pOD->pz_DisableName );
-            }
+
+            /*
+             *  IF the option defaults to being enabled,
+             *  THEN print that out
+             */
+            if (pOD->fOptState & OPTST_INITENABLED)
+                fputs( zEnab, fp );
 
             /*
              *  IF  the option is in an equivalence class
@@ -256,42 +293,29 @@ optionUsage( tOptions*  pOptions, int exitCode )
              */
             if (  (pOD->optEquivIndex != NO_EQUIVALENT)
                && (pOD->optEquivIndex != optNo )  )  {
-                tSCC zAlt[] = "\t\t\t\t- an alternate for %s\n";
                 fprintf( fp, zAlt,
                          pOptions->pOptDesc[ pOD->optEquivIndex ].pz_Name );
                 continue;
             }
 
-            if ((pOD->fOptState & OPTST_NO_INIT) != 0) {
-                if (  ((pOptions->fOptSet & OPTPROC_EXERC) != 0)
-                   || (pOptions->papzHomeList != (const char**)NULL)
-                   || (pOptions->pzPROGNAME != (const char*)NULL)  ) {
-                    tSCC zNoPreset[] = "\t\t\t\t- may not be preset\n";
-                    fputs( zNoPreset, fp );
-                }
-            }
+            /*
+             *  IF this particular option can NOT be preset
+             *    AND some form of presetting IS allowed,
+             *  THEN advise that this option may not be preset.
+             */
+            if (  ((pOD->fOptState & OPTST_NO_INIT) != 0)
+               && (  (pOptions->papzHomeList != (const char**)NULL)
+                  || (pOptions->pzPROGNAME != (const char*)NULL)
+               )  )
+
+                fputs( zNoPreset, fp );
 
             /*
              *  Print the appearance requirements.
              */
             switch (pOD->optMinCt) {
             case 1:
-            {
-                /*
-                 *  One is required.
-                 *  We mention maximums in the next case element.
-                 */
-                tSCC zReqFlag[] = "\t\t\t\t- a required option\n";
-                fputs( zReqFlag, fp );
-                /*FALLTHROUGH*/
-            }
-
             case 0:
-            {
-                tSCC zNoLim[]  = "\t\t\t\t- may appear without limit\n";
-                tSCC zPreset[] = "\t\t\t\t- may NOT appear - preset only\n";
-                tSCC zUpTo[]   = "\t\t\t\t- may appear up to %d times\n";
-
                 /*
                  *  IF the max is more than one, print an "UP TO" message
                  */
@@ -312,78 +336,53 @@ optionUsage( tOptions*  pOptions, int exitCode )
                      break;
                 }
                 break;
-            }
 
             default:
-            {
-                tSCC zMust[] = "\t\t\t\t- must appear between %d and "
-                               "%d times\n";
                 /*
                  *  More than one is required.  Print the range.
                  */
                 fprintf( fp, zMust, pOD->optMinCt, pOD->optMaxCt );
             }
-            }
 
             if (  NAMED_OPTS( pOptions )
                && (pOptions->specOptIdx.default_opt == pOD->optIndex))
-                fputs( "\t\t\t\t- default option for unnamed options\n",
-                       fp );
+                fputs( zDefaultOpt, fp );
         }  while (pOD++, optNo++, (--ct > 0));
     }
 
     fputc( '\n', fp );
 
+    /*
+     *  Describe the mechanics of denoting the options
+     */
     {
-        tSCC zOptsOnly[]  = "All arguments are named options.\n";
-        tSCC zInverted[]  = "Flag options are disabled with a '+' marker.\n";
-        tSCC zNumberOpt[] = "The '-#<number>' option may omit the hash char\n";
+        tCC*   pzFmt   =  (pOptions->fOptSet & OPTPROC_SHORTOPT)
+                          ? zFlagOkay : zNoFlags;
 
-        tSCC zFlagOkay[]    =
-            "Options may be specified by doubled %1$ss and their name\n"
-            "or by a single %1$s and the flag character/option value.\n";
+        if ((pOptions->fOptSet & OPTPROC_LONGOPT) != 0)
+            fputs( pzFmt, fp );
 
-        tSCC zNoFlags[]     =
-            "Options are specified by their name and either single\n"
-            "or doubled %ss.  Flag characters are not interpreted.\n";
-
-        u_int  fOptSet = pOptions->fOptSet;
-        tCC*   pzFmt   =  (fOptSet & OPTPROC_SHORTOPT) ? zFlagOkay : zNoFlags;
-
-        if ((fOptSet & OPTPROC_PLUSMARKS) != 0)
-            fputs( zInverted, fp );
-
-        if ((fOptSet & OPTPROC_LONGOPT) != 0)
-            fprintf( fp, pzFmt,
-                     (fOptSet & OPTPROC_DISABLEOK) ? "marker" : "hyphen" );
-
-        else if ((fOptSet & OPTPROC_SHORTOPT) == 0)
+        else if ((pOptions->fOptSet & OPTPROC_SHORTOPT) == 0)
             fputs( zOptsOnly, fp );
 
-        if ((fOptSet & OPTPROC_NUM_OPT) != 0)
+        if ((pOptions->fOptSet & OPTPROC_NUM_OPT) != 0)
             fputs( zNumberOpt, fp );
     }
 
     if (pOptions->pzExplain != (char*)NULL)
         fputs( pOptions->pzExplain, fp );
 
-
+    /*
+     *  IF the user is asking for help (thus exiting with SUCCESS),
+     *  THEN see what additional information we can provide.
+     */
     if (exitCode == EXIT_SUCCESS) {
         ag_bool  initIntro = AG_TRUE;
-        u_int    fOptSet   = pOptions->fOptSet;
-        tSCC     zIntro[]  = "\nThe following option preset mechanisms "
-                             "are supported:\n";
         tSCC     zPathFmt[] = " - reading file %s/%s\n";
 
-        if ((fOptSet & OPTPROC_EXERC) != 0) {
-            tSCC zHomePath[] = " - reading file /... %s's exe directory "
-                               ".../%s \n";
-            fputs( zIntro, fp );
-            fprintf( fp, zHomePath, pOptions->pzProgName,
-                     pOptions->pzRcName );
-            initIntro = AG_FALSE;
-        }
-
+        /*
+         *  Display all the places we look for RC files
+         */
         if (pOptions->papzHomeList != (const char**)NULL) {
             const char** papzHL = pOptions->papzHomeList;
             for (;;) {
@@ -401,84 +400,45 @@ optionUsage( tOptions*  pOptions, int exitCode )
             }
         }
 
+        /*
+         *  Let the user know about environment variable settings
+         */
         if ((pOptions->fOptSet & OPTPROC_ENVIRON) != 0) {
-            tSCC zExamineFmt[] = " - examining environment variables "
-                                 "named %s_*\n";
             if (initIntro)
                 fputs( zIntro, fp );
 
             fprintf( fp, zExamineFmt, pOptions->pzPROGNAME );
         }
 
-        if (pOptions->pzDetail != (char*)NULL)
-            fputs( pOptions->pzDetail, fp );
+        /*
+         *  IF we found an enumeration,
+         *  THEN hunt for it again.  Call the handler proc with a NULL
+         *       option struct pointer.  That tells it to display the keywords.
+         */
+        if (displayEnum) {
+            int        ct     = pOptions->optCt;
+            int        optNo  = 0;
+            tOptDesc*  pOD    = pOptions->pOptDesc;
+
+            fputc( '\n', fp );
+            fflush( fp );
+            do  {
+                if ((pOD->fOptState & OPTST_ENUMERATION) != 0)
+                    (*(pOD->pOptProc))( NULL, pOD );
+            }  while (pOD++, optNo++, (--ct > 0));
+        }
 
         /*
-         *  IF      there is a detail file
-         *     AND  we are using the pager
-         *  THEN uncompress the detail file and dump to fp
+         *  If there is a detail string, now is the time for that.
          */
-        if (  (pOptions->pzDetailFile != (char*)NULL)
-           && (! UNUSED_OPT( pOptions->pOptDesc
-                           + pOptions->specOptIdx.more_help ))) {
-            char* pzPath = getenv( "PATH" );
-            char* pzHelpPath;
-
-            char  zCmd[ MAXPATHLEN + 64 ];
-            char  zDetFile[ MAXPATHLEN ];
-
-            do {
-                char*  pz = zDetFile
-                    + snprintf( zDetFile, sizeof(zDetFile), "%s.README",
-                                pOptions->pzDetailFile );
-                pzHelpPath = pathfind( pzPath, zDetFile, "r" );
-
-                if (pzHelpPath != (char*)NULL) {
-                    tSCC zCmdFmt[] = "cat %s >&2";
-                    snprintf( zCmd, sizeof(zCmd), zCmdFmt, pzHelpPath );
-                    break;
-                }
-
-                strcpy( pz, ".Z" );
-                pzHelpPath = pathfind( pzPath, zDetFile, "r" );
-
-                if (pzHelpPath != (char*)NULL) {
-                    tSCC zCmdFmt[] = "uncompress -c < %s >&2";
-                    snprintf( zCmd, sizeof(zCmd), zCmdFmt, pzHelpPath );
-                    break;
-                }
-
-                strcpy( pz, ".gz" );
-                pzHelpPath = pathfind( pzPath, zDetFile, "r" );
-
-                if (pzHelpPath != (char*)NULL) {
-                    tSCC zCmdFmt[] = "gunzip -c %s >&2";
-                    snprintf( zCmd, sizeof(zCmd), zCmdFmt, pzHelpPath );
-                    break;
-                }
-
-                strcpy( pz, ".bz2" );
-                pzHelpPath = pathfind( pzPath, zDetFile, "r" );
-
-                if (pzHelpPath != (char*)NULL) {
-                    tSCC zCmdFmt[] = "bunzip2 -c %s >&2";
-                    snprintf( zCmd, sizeof(zCmd), zCmdFmt, pzHelpPath );
-                    break;
-                }
-
-            } while (0);
-
-            /*
-             *  IF we can actually find the detail file,
-             *  THEN really do it.
-             */
-            if (pzHelpPath != (char*)NULL) {
-                fflush( fp );
-                system( zCmd );
-            }
-        }
+        if (pOptions->pzDetail != (char*)NULL)
+            fputs( pOptions->pzDetail, fp );
     }
 
     exit( exitCode );
 }
-/* usage.c ends here */
+/*
+ * Local Variables:
+ * c-file-style: "stroustrup"
+ * End:
+ * usage.c ends here */
